import json
import re
from typing import List

from bs4 import Tag

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import Job, Identifier, Location
from pyethiojobs.utils import remove_newline


class GetLatestJobs(Scaffold):
    async def get_latest_jobs(self) -> List[Job]:
        """
        Get the latest jobs from the site.
        """
        res = await self._process_request("jobs-in-ethiopia/")
        soup = self.soup(res.text)
        job_container = soup.find("div", {"class": "content_latest marg-1"})
        listing = job_container.find_all("div", {"class": "single_listing col-md-6"})
        json_scripts: List[Tag] = job_container.find_all(
            "script", {"type": "application/ld+json"}
        )
        latest_jobs = []
        for job, single_list in zip(json_scripts, listing):
            link = single_list.find("a")
            text = re.sub(r'\w+\s+"(\w+)"\s+\w+', r" '\1' ", remove_newline(job.text))
            job = json.loads(text)
            experience = (
                re.search(
                    r".+\s*Required Experience:\s*(?:<\/\w+>|)(.+\s*.+)(?:\s*<\w+>|)",
                    job["description"],
                )
                .group(1)
                .strip()
            )
            latest_jobs.append(
                Job(
                    job_id=re.search(r"\/(\d+)\/", link["href"]).group(1),
                    title=job["title"].strip(),
                    description=job["description"],
                    identifier=Identifier(
                        name=job["identifier"]["name"],
                        id=job["identifier"]["value"],
                    ),
                    experience=experience,
                    location=Location(
                        address_locality=job["jobLocation"]["address"][
                            "addressLocality"
                        ].strip(),
                        address_region=job["jobLocation"]["address"][
                            "addressRegion"
                        ].strip(),
                        address_country=job["jobLocation"]["address"][
                            "addressCountry"
                        ].strip(),
                    ),
                    date_posted=job["datePosted"],
                    valid_through=job["validThrough"],
                    type=job.get("employmentType"),
                    hiring_organization=job["hiringOrganization"]["name"].strip(),
                    link=link["href"],
                    base=self,
                    work_place=None,
                )
            )
        return latest_jobs



from pyethiojobs import types


class JobDetails:
    def __init__(
        self,
        id: int,
        title: str,
        description: str,
        link: str,
        views: int,
        category: str,
        salary: str,
        print_link: str,
        identifier: "types.Identifier",
        date_posted: str,
        valid_through: str,
        hiring_organization: str,
        location: "types.Location",
        work_place: str,
        type: str,
        career_level: str,
    ):
        self.id = id
        self.print_link = print_link
        self.salary = salary
        self.category = category
        self.views = views
        self.work_place = work_place
        self.title = title
        self.description = description
        self.link = link
        self.identifier = identifier
        self.date_posted = date_posted
        self.valid_through = valid_through
        self.hiring_organization = hiring_organization
        self.location = location
        self.type = type
        self.career_level = career_level



from typing import Union

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import Job, JobDetails


class GetJob(Scaffold):
    async def get_job(self, job: Union[int, str, Job]) -> JobDetails:
        link = (
            job.link
            if isinstance(job, Job)
            else self._BASE_URL.format("display-job/" + job)
        )
        res = await self._process_request(url=link)
        return self._get_job_details(res.text)



from typing import Union

import pyethiojobs
from pyethiojobs import types


class GovJob:
    def __init__(
        self,
        company: str,
        link: str,
        job: str,
        posted: str,
        base: "pyethiojobs.pyEthioJobs" = None,
    ):
        self.company = company
        self.link = link
        self.job = job
        self.posted = posted
        self._base = base

    async def get_details(self) -> Union["types.GovJobDetails", None]:
        if self._base is not None:
            res = await self._base._process_request(url=self.link)
            return self._base._get_gov_job_details(res.text)



from typing import List, Callable, Union

import httpx
from bs4 import BeautifulSoup

from pyethiojobs.filters import Filter
from pyethiojobs.handler import HandlersHolder
from pyethiojobs.types import Job, JobDetails, EmploymentType, GovJobDetails, Category
from pyethiojobs.types.jobs import GovJob


class Scaffold:
    def __init__(self):
        self._session = None
        self._soup = None
        self._BASE_URL = "https://www.ethiojobs.net/{}"
        self._on_event_update: HandlersHolder = None

    async def _process_request(
        self, url: str, method="GET", **kwargs
    ) -> httpx.Response:
        """
        Process a request.
        :param url: URL to request.
        :param method: Method to use. Default is GET.
        :param kwargs: Keyword arguments to pass to the request.
        :return:
        """
        pass

    def soup(self, html: str) -> BeautifulSoup:
        """
        Get a BeautifulSoup object from an HTML string.
        :param html: HTML string.
        :return:
        """
        pass

    def _get_jobs(self, html) -> List[Job]:
        """
        Get jobs from HTML.
        :param html:
        :return:
        """
        pass

    def _get_job_details(self, html) -> JobDetails:
        """
        Get job details from HTML.
        :param html:
        :return:
        """
        pass

    async def get_categories(self) -> List[Category]:
        """
        Get all categories from the site.
        :return: List of categories.
        """
        pass

    async def get_category(
        self, category: Union[int, str, Category], limit: int = 10
    ) -> List[Job]:
        """
        Get jobs from a category.
        :param category: Category to get jobs from. Can be a Category object, a string or an int.
        :param limit: Number of jobs to get. Default is 10.
        :return: List of jobs.
        """
        pass

    def on_new_jobs(self, filters: Filter = None) -> Callable:
        """
        Decorator to listen for new jobs.
        :param filters: Filters to apply to the jobs.
        :return: Decorator.
        """
        pass

    def _get_gov_job_details(self, html) -> GovJobDetails:
        pass

    async def get_gov_jobs(self) -> List[GovJob]:
        """
        Get the latest government jobs from the site.
        :return: List of Gov jobs.
        """
        pass

    async def get_latest_jobs(self) -> List[Job]:
        """
        Get the latest jobs from the site.
        :return: List of jobs.
        """
        pass

    async def search(
        self, query: str, limit: int = 10, employment_type: EmploymentType = None
    ) -> List[Job]:
        """
        Search for jobs.
        :param query: Query to search for.
        :param limit: Number of jobs to return. Default is 10.
        :param employment_type: Type of employment to search for. Default is None. Can be FULL_TIME, PART_TIME,
        CONTRACT, INTERNSHIP, VOLUNTEER.
        :return: List of jobs.
        """
        pass

    def run(self, poll_interval: int = 60) -> None:
        """
        Run the listener.
        :param poll_interval: Interval to poll for new jobs. Default is 60 seconds.
        """
        pass



from typing import Union

from pyethiojobs import types


class Job:
    def __init__(
        self,
        job_id: str,
        title: str,
        description: str,
        link: str,
        identifier: "types.Identifier",
        experience: str,
        date_posted: str,
        valid_through: str,
        hiring_organization: str,
        location: "types.Location",
        work_place: Union[str, None],
        type: str,
        base: "pyethiojobs.pyEthioJobs" = None,
    ):
        self.job_id = job_id
        self.experience = experience
        self.work_place = work_place
        self.title = title
        self.description = description
        self.link = link
        self.identifier = identifier
        self.date_posted = date_posted
        self.valid_through = valid_through
        self.hiring_organization = hiring_organization
        self.location = location
        self._base = base
        self.type = type

    async def get_details(self):
        if self._base is not None:
            res = await self._base._process_request(url=self.link)
            return self._base._get_job_details(res.text)

    def __repr__(self):
        return f"Job(title={self.title}, link={self.link})"

    def __str__(self):
        return f"Job(title={self.title}, link={self.link})"

    def __eq__(self, other):
        return self.title == other.title and self.link == other.link



import asyncio

from rich.pretty import pprint

from pyethiojobs.scaffold import Scaffold


class Run(Scaffold):
    def run(self, poll_interval: int = 60) -> None:
        """
        Run the scraper.
        """
        loop = asyncio.get_event_loop()
        try:
            loop.run_until_complete(self._run(poll_interval))
            loop.close()
        except KeyboardInterrupt:
            print("Exiting...")
            loop.close()

    async def _run(self, poll_interval):
        """
        Run the scraper.
        """

        latest_jobs = await self.get_latest_jobs()
        first_job = latest_jobs[0]
        pprint(latest_jobs)
        while True:
            print("Checking for new jobs...")
            new_jobs = await self.get_latest_jobs()
            if new_jobs[0] != first_job:
                print("New job posted!")
                first_job = new_jobs[0]
                index_of_first_job = new_jobs.index(first_job)
                new_jobs = new_jobs[:index_of_first_job]
                for job in new_jobs:
                    pprint(job)
                    await self._on_event_update.incoming_job_handler(job=job)

            await asyncio.sleep(poll_interval)



import inspect
from typing import Callable, Optional, List, Dict

from pyethiojobs.types import Job


class HandlersHolder:
    def __init__(self, base):
        self._base = base
        self._events: Dict[str, List] = {
            "NEW_JOB_HANDLER": [],
        }

    def set_handler(
        self, event_name: str, func: Callable, filters: Optional[Callable] = None
    ):
        if inspect.iscoroutinefunction(func):
            return self._events[event_name].append({"func": func, "filters": filters})
        raise TypeError(f"{func.__name__} is not a coroutine function")

    def remove_handler(self, event_name: str, func: Callable):
        self._events[event_name] = [
            event for event in self._events[event_name] if event["func"] != func
        ]

    async def incoming_job_handler(self, job: Job):
        for event in self._events["NEW_JOB_HANDLER"]:
            if event["filters"] is not None:
                if event["filters"](job):
                    await event["func"](job)
            else:
                await event["func"](job)



import inspect
from datetime import datetime, timedelta
from typing import Callable, Union, List

from pyethiojobs.types import Job


class Filter:
    async def __call__(self, job: Job):
        raise NotImplementedError

    def __invert__(self):
        return Invert(self)

    def __and__(self, other):
        return And(self, other)

    def __or__(self, other):
        return Or(self, other)


class Invert(Filter):
    def __init__(self, base):
        self.base = base

    async def __call__(self, job: Job):
        if inspect.iscoroutinefunction(self.base.__call__):
            x = await self.base(job)
            return not x
        return


class And(Filter):
    def __init__(self, base, other):
        self.base = base
        self.other = other

    async def __call__(self, job: Job):
        x = await self.base(job)

        if not x:
            return False

        y = await self.other(job)

        return x and y


class Or(Filter):
    def __init__(self, base, other):
        self.base = base
        self.other = other

    async def __call__(self, job: Job):
        x = await self.base(job)
        if x:
            return True

        y = await self.other(job)

        return x or y


CUSTOM_FILTER_NAME = "MyCustomFilter"


def create(func: Callable, name: str = None, **kwargs) -> Filter:
    return type(
        name or func.__name__ or CUSTOM_FILTER_NAME,
        (Filter,),
        {"__call__": func, **kwargs},
    )()


def work_place(places: Union[str, List[str]]):
    async def func(flt, job: Job):
        if job.work_place in flt.places:
            return True

    return create(
        func, "PlacesFilter", place=places if isinstance(places, list) else [places]
    )


def date(dates: Union[str, List[str]]):
    async def func(flt, job: Job):
        if job.date_posted in flt.dates:
            return True

    return create(
        func, "DateFilter", dates=dates if isinstance(dates, list) else [dates]
    )


def valid_through(days: Union[str, int]):
    async def func(flt, job: Job):
        today = datetime.today()
        job_valid_through = datetime.strptime(job.valid_through, "%Y-%m-%d %H:%M:%S")
        if job_valid_through < today + timedelta(days=flt.days):
            return True

    return create(func, "ValidThroughFilter", days=days)


def hiring_organization(organizations: Union[str, List[str]]):
    async def func(flt, job: Job):
        if job.hiring_organization in flt.organizations:
            return True

    return create(
        func,
        "HiringOrganizationFilter",
        organizations=organizations
        if isinstance(organizations, list)
        else [organizations],
    )


def location(
    address_country: Union[str, List[str]] = "ETH",
    address_region: Union[str, List[str]] = None,
    address_locality: Union[str, List[str]] = None,
):
    async def func(flt, job: Job):
        if job.location.address_country in flt.address_country:
            if flt.address_region is not None:
                if job.location.address_region in flt.address_region:
                    if flt.address_locality is not None:
                        if job.location.address_locality in flt.address_locality:
                            return True
                    else:
                        return True
            else:
                return True

    return create(
        func,
        "LocationFilter",
        address_country=address_country,
        address_region=address_region,
        address_locality=address_locality,
    )



from pyethiojobs import types


class GovJobDetails:
    def __init__(
        self,
        job_title: str,
        company: str,
        location: str,
        gov_type: str,
        deadline: str,
        job_type: "types.JobType",
        description: str,
    ):
        self.job_title = job_title
        self.job_type = job_type
        self.company = company
        self.location = location
        self.gov_type = gov_type
        self.deadline = deadline
        self.description = description



from typing import Union, List

from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import Category, Job


class GetCategory(Scaffold):
    async def get_category(
        self, category: Union[int, str, Category], limit: int = 10
    ) -> List[Job]:
        link = (
            category.link
            if isinstance(category, Category)
            else self._BASE_URL.format("browse-by-category/" + category)
        )
        res = await self._process_request(url=link, params={"listings_per_page": limit})
        return self._get_jobs(res.text)



from .gov_job import GovJob
from .gov_job_details import GovJobDetails
from .identifier import Identifier
from .job import Job
from .job_details import JobDetails
from .job_type import JobType
from .location import Location



from typing import List

from pyethiojobs import types
from pyethiojobs.scaffold import Scaffold
from pyethiojobs.types import Job


class GetSearch(Scaffold):
    async def search(
        self, query: str, limit: int = 10, employment_type: types.EmploymentType = None
    ) -> List[Job]:
        url = self._BASE_URL.format("search-results-jobs/")
        params = {
            "action": "search",
            "listings_per_page": limit,
            "listing_type[equal]": "Job",
            "keywords[all_words]": query,
            "EmploymentType[multi_like][]": employment_type.value
            if employment_type
            else None,
        }
        res = await self._process_request(url, params=params)
        return self._get_jobs(res.text)

    async def search_company(self, company: str):
        pass


# https://www.ethiojobs.net/search-results-jobs/?action=search&listing_type[equal]=Job&keywords[all_words]=Graphics Designer&view=list&EmploymentType[multi_like][]=76
